# TODO - remove this special case when we fix the generated code for empty openapi structs
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

import attr

if TYPE_CHECKING:
    from ..models.validation_error import ValidationError


T = TypeVar("T", bound="HTTPValidationError")


@attr.s(auto_attribs=True)
class HTTPValidationError(Exception):
    """
    Attributes:
        detail (List['ValidationError']):
    """

    detail: List["ValidationError"]
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    status_code: int = 0

    @classmethod
    def init_exception(cls: Type[T], response: Dict[str, str], status_code: int) -> T:
        ret = cls.from_dict(response)
        ret.status_code = status_code
        return ret

    def to_dict(self) -> Dict[str, Any]:
        detail = []
        for detail_item_data in self.detail:
            detail_item = detail_item_data.to_dict()

            detail.append(detail_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "detail": detail,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.validation_error import ValidationError

        d = src_dict.copy()
        detail = []
        _detail = d.pop("detail")
        for detail_item_data in _detail:
            detail_item = ValidationError.from_dict(detail_item_data)

            detail.append(detail_item)

        http_validation_error = cls(
            detail=detail,
        )

        http_validation_error.additional_properties = d
        return http_validation_error

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import asyncio
import random
from http import HTTPStatus
from time import sleep
from typing import Any, Dict, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.environment_out import EnvironmentOut
from ...models.http_error import HttpError
from ...models.http_validation_error import HTTPValidationError
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: AuthenticatedClient,
    idempotency_key: Union[Unset, str] = UNSET,
) -> Dict[str, Any]:
    url = "{}/api/v1/environment/export".format(client.base_url)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    if not isinstance(idempotency_key, Unset):
        headers["idempotency-key"] = idempotency_key

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "follow_redirects": client.follow_redirects,
    }


def _parse_response(*, client: Client, response: httpx.Response) -> EnvironmentOut:
    if response.status_code == HTTPStatus.OK:
        response_200 = EnvironmentOut.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.BAD_REQUEST:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.FORBIDDEN:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.NOT_FOUND:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.CONFLICT:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.UNPROCESSABLE_ENTITY:
        raise HTTPValidationError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.TOO_MANY_REQUESTS:
        raise HttpError.init_exception(response.json(), response.status_code)
    raise errors.UnexpectedStatus(response.status_code, response.content)


def _build_response(*, client: Client, response: httpx.Response) -> Response[EnvironmentOut]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def request_sync_detailed(
    *,
    client: AuthenticatedClient,
    idempotency_key: Union[Unset, str] = UNSET,
) -> Response[EnvironmentOut]:
    """Export Environment Configuration

     Download a JSON file containing all org-settings and event types

    Args:
        idempotency_key (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[EnvironmentOut]
    """

    kwargs = _get_kwargs(
        client=client,
        idempotency_key=idempotency_key,
    )

    kwargs["headers"] = {"svix-req-id": f"{random.getrandbits(32)}", **kwargs["headers"]}

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )
    for retry_count, sleep_time in enumerate(client.retry_schedule):
        if response.status_code < 500:
            break

        sleep(sleep_time)
        kwargs["headers"]["svix-retry-count"] = str(retry_count)
        response = httpx.request(
            verify=client.verify_ssl,
            **kwargs,
        )

    return _build_response(client=client, response=response)


def request_sync(
    *,
    client: AuthenticatedClient,
    idempotency_key: Union[Unset, str] = UNSET,
) -> EnvironmentOut:
    """Export Environment Configuration

     Download a JSON file containing all org-settings and event types

    Args:
        idempotency_key (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        EnvironmentOut
    """

    return request_sync_detailed(
        client=client,
        idempotency_key=idempotency_key,
    ).parsed


async def request_asyncio_detailed(
    *,
    client: AuthenticatedClient,
    idempotency_key: Union[Unset, str] = UNSET,
) -> Response[EnvironmentOut]:
    """Export Environment Configuration

     Download a JSON file containing all org-settings and event types

    Args:
        idempotency_key (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[EnvironmentOut]
    """

    kwargs = _get_kwargs(
        client=client,
        idempotency_key=idempotency_key,
    )

    kwargs["headers"] = {"svix-req-id": f"{random.getrandbits(32)}", **kwargs["headers"]}

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

        for retry_count, sleep_time in enumerate(client.retry_schedule):
            if response.status_code < 500:
                break

            await asyncio.sleep(sleep_time)
            kwargs["headers"]["svix-retry-count"] = str(retry_count)
            response = await _client.request(**kwargs)

    return _build_response(client=client, response=response)


async def request_asyncio(
    *,
    client: AuthenticatedClient,
    idempotency_key: Union[Unset, str] = UNSET,
) -> EnvironmentOut:
    """Export Environment Configuration

     Download a JSON file containing all org-settings and event types

    Args:
        idempotency_key (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        EnvironmentOut
    """

    return (
        await request_asyncio_detailed(
            client=client,
            idempotency_key=idempotency_key,
        )
    ).parsed



# TODO - remove this special case when we fix the generated code for empty openapi structs
from typing import Any, Dict, List, Type, TypeVar

import attr

from ..models.sink_out_type_0_type import SinkOutType0Type

T = TypeVar("T", bound="SinkOutType0")


@attr.s(auto_attribs=True)
class SinkOutType0:
    """
    Attributes:
        routing_key (str):
        type (SinkOutType0Type):
        uri (str):
    """

    routing_key: str
    type: SinkOutType0Type
    uri: str
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        routing_key = self.routing_key
        type = self.type.value

        uri = self.uri

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "routingKey": routing_key,
                "type": type,
                "uri": uri,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        routing_key = d.pop("routingKey")

        type = SinkOutType0Type(d.pop("type"))

        uri = d.pop("uri")

        sink_out_type_0 = cls(
            routing_key=routing_key,
            type=type,
            uri=uri,
        )

        sink_out_type_0.additional_properties = d
        return sink_out_type_0

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



""" Contains all the data models used in inputs/outputs """

from .aggregate_event_types_out import AggregateEventTypesOut
from .app_portal_access_in import AppPortalAccessIn
from .app_portal_access_out import AppPortalAccessOut
from .app_usage_stats_in import AppUsageStatsIn
from .app_usage_stats_out import AppUsageStatsOut
from .application_in import ApplicationIn
from .application_in_metadata import ApplicationInMetadata
from .application_out import ApplicationOut
from .application_out_metadata import ApplicationOutMetadata
from .application_patch import ApplicationPatch
from .application_patch_metadata import ApplicationPatchMetadata
from .application_stats import ApplicationStats
from .application_token_expire_in import ApplicationTokenExpireIn
from .attempt_statistics_data import AttemptStatisticsData
from .attempt_statistics_response import AttemptStatisticsResponse
from .auth_token_out import AuthTokenOut
from .background_task_data import BackgroundTaskData
from .background_task_out import BackgroundTaskOut
from .background_task_status import BackgroundTaskStatus
from .background_task_type import BackgroundTaskType
from .border_radius_config import BorderRadiusConfig
from .border_radius_enum import BorderRadiusEnum
from .client_secret_jwt_params_in import ClientSecretJwtParamsIn
from .completion_choice import CompletionChoice
from .completion_message import CompletionMessage
from .create_message_token_in import CreateMessageTokenIn
from .create_stream_in import CreateStreamIn
from .create_stream_out import CreateStreamOut
from .custom_color_palette import CustomColorPalette
from .custom_strings_override import CustomStringsOverride
from .custom_theme_override import CustomThemeOverride
from .dashboard_access_out import DashboardAccessOut
from .duration import Duration
from .endpoint_created_event import EndpointCreatedEvent
from .endpoint_created_event_data import EndpointCreatedEventData
from .endpoint_created_event_type import EndpointCreatedEventType
from .endpoint_deleted_event import EndpointDeletedEvent
from .endpoint_deleted_event_data import EndpointDeletedEventData
from .endpoint_deleted_event_type import EndpointDeletedEventType
from .endpoint_disabled_event import EndpointDisabledEvent
from .endpoint_disabled_event_data import EndpointDisabledEventData
from .endpoint_disabled_event_type import EndpointDisabledEventType
from .endpoint_headers_in import EndpointHeadersIn
from .endpoint_headers_in_headers import EndpointHeadersInHeaders
from .endpoint_headers_out import EndpointHeadersOut
from .endpoint_headers_out_headers import EndpointHeadersOutHeaders
from .endpoint_headers_patch_in import EndpointHeadersPatchIn
from .endpoint_headers_patch_in_headers import EndpointHeadersPatchInHeaders
from .endpoint_in import EndpointIn
from .endpoint_in_metadata import EndpointInMetadata
from .endpoint_message_out import EndpointMessageOut
from .endpoint_message_out_payload import EndpointMessageOutPayload
from .endpoint_mtls_config_in import EndpointMtlsConfigIn
from .endpoint_oauth_config_in import EndpointOauthConfigIn
from .endpoint_oauth_config_in_extra_params import EndpointOauthConfigInExtraParams
from .endpoint_out import EndpointOut
from .endpoint_out_metadata import EndpointOutMetadata
from .endpoint_patch import EndpointPatch
from .endpoint_patch_metadata import EndpointPatchMetadata
from .endpoint_secret_out import EndpointSecretOut
from .endpoint_secret_rotate_in import EndpointSecretRotateIn
from .endpoint_stats import EndpointStats
from .endpoint_transformation_in import EndpointTransformationIn
from .endpoint_transformation_out import EndpointTransformationOut
from .endpoint_transformation_simulate_in import EndpointTransformationSimulateIn
from .endpoint_transformation_simulate_in_payload import EndpointTransformationSimulateInPayload
from .endpoint_transformation_simulate_out import EndpointTransformationSimulateOut
from .endpoint_update import EndpointUpdate
from .endpoint_update_metadata import EndpointUpdateMetadata
from .endpoint_updated_event import EndpointUpdatedEvent
from .endpoint_updated_event_data import EndpointUpdatedEventData
from .endpoint_updated_event_type import EndpointUpdatedEventType
from .environment_in import EnvironmentIn
from .environment_out import EnvironmentOut
from .environment_settings_out import EnvironmentSettingsOut
from .event_example_in import EventExampleIn
from .event_in import EventIn
from .event_out import EventOut
from .event_stream_out import EventStreamOut
from .event_type_example_out import EventTypeExampleOut
from .event_type_example_out_example import EventTypeExampleOutExample
from .event_type_example_out_example_additional_property import EventTypeExampleOutExampleAdditionalProperty
from .event_type_from_open_api import EventTypeFromOpenApi
from .event_type_from_open_api_schemas import EventTypeFromOpenApiSchemas
from .event_type_from_open_api_schemas_additional_property import EventTypeFromOpenApiSchemasAdditionalProperty
from .event_type_import_open_api_in import EventTypeImportOpenApiIn
from .event_type_import_open_api_in_spec import EventTypeImportOpenApiInSpec
from .event_type_import_open_api_in_spec_additional_property import EventTypeImportOpenApiInSpecAdditionalProperty
from .event_type_import_open_api_out import EventTypeImportOpenApiOut
from .event_type_import_open_api_out_data import EventTypeImportOpenApiOutData
from .event_type_in import EventTypeIn
from .event_type_in_schemas import EventTypeInSchemas
from .event_type_in_schemas_additional_property import EventTypeInSchemasAdditionalProperty
from .event_type_out import EventTypeOut
from .event_type_out_schemas import EventTypeOutSchemas
from .event_type_out_schemas_additional_property import EventTypeOutSchemasAdditionalProperty
from .event_type_patch import EventTypePatch
from .event_type_patch_schemas import EventTypePatchSchemas
from .event_type_patch_schemas_additional_property import EventTypePatchSchemasAdditionalProperty
from .event_type_schema_in import EventTypeSchemaIn
from .event_type_schema_in_schema import EventTypeSchemaInSchema
from .event_type_update import EventTypeUpdate
from .event_type_update_schemas import EventTypeUpdateSchemas
from .event_type_update_schemas_additional_property import EventTypeUpdateSchemasAdditionalProperty
from .export_event_type_out import ExportEventTypeOut
from .font_size_config import FontSizeConfig
from .generate_in import GenerateIn
from .generate_out import GenerateOut
from .http_error import HttpError
from .http_validation_error import HTTPValidationError
from .hubspot_oauth_config_in import HubspotOauthConfigIn
from .inbound_path_params import InboundPathParams
from .incoming_webhook_payload_out import IncomingWebhookPayloadOut
from .integration_in import IntegrationIn
from .integration_key_out import IntegrationKeyOut
from .integration_out import IntegrationOut
from .integration_update import IntegrationUpdate
from .kafka_security_protocol_type import KafkaSecurityProtocolType
from .list_response_application_out import ListResponseApplicationOut
from .list_response_application_stats import ListResponseApplicationStats
from .list_response_background_task_out import ListResponseBackgroundTaskOut
from .list_response_endpoint_message_out import ListResponseEndpointMessageOut
from .list_response_endpoint_out import ListResponseEndpointOut
from .list_response_event_type_out import ListResponseEventTypeOut
from .list_response_integration_out import ListResponseIntegrationOut
from .list_response_message_attempt_endpoint_out import ListResponseMessageAttemptEndpointOut
from .list_response_message_attempt_out import ListResponseMessageAttemptOut
from .list_response_message_endpoint_out import ListResponseMessageEndpointOut
from .list_response_message_out import ListResponseMessageOut
from .list_response_sink_out import ListResponseSinkOut
from .list_response_stream_out import ListResponseStreamOut
from .list_response_stream_sink_out import ListResponseStreamSinkOut
from .list_response_template_out import ListResponseTemplateOut
from .message_attempt_endpoint_out import MessageAttemptEndpointOut
from .message_attempt_exhausted_event import MessageAttemptExhaustedEvent
from .message_attempt_exhausted_event_data import MessageAttemptExhaustedEventData
from .message_attempt_exhausted_event_type import MessageAttemptExhaustedEventType
from .message_attempt_failed_data import MessageAttemptFailedData
from .message_attempt_failing_event import MessageAttemptFailingEvent
from .message_attempt_failing_event_data import MessageAttemptFailingEventData
from .message_attempt_failing_event_type import MessageAttemptFailingEventType
from .message_attempt_headers_out import MessageAttemptHeadersOut
from .message_attempt_headers_out_sent_headers import MessageAttemptHeadersOutSentHeaders
from .message_attempt_out import MessageAttemptOut
from .message_attempt_recovered_event import MessageAttemptRecoveredEvent
from .message_attempt_recovered_event_data import MessageAttemptRecoveredEventData
from .message_attempt_recovered_event_type import MessageAttemptRecoveredEventType
from .message_attempt_trigger_type import MessageAttemptTriggerType
from .message_broadcast_in import MessageBroadcastIn
from .message_broadcast_in_payload import MessageBroadcastInPayload
from .message_broadcast_out import MessageBroadcastOut
from .message_endpoint_out import MessageEndpointOut
from .message_events_out import MessageEventsOut
from .message_in import MessageIn
from .message_in_payload import MessageInPayload
from .message_in_transformations_params import MessageInTransformationsParams
from .message_out import MessageOut
from .message_out_payload import MessageOutPayload
from .message_raw_payload_out import MessageRawPayloadOut
from .message_status import MessageStatus
from .message_subscriber_auth_token_out import MessageSubscriberAuthTokenOut
from .o_auth_payload_in import OAuthPayloadIn
from .o_auth_payload_out import OAuthPayloadOut
from .oauth_2_auth_method_in import Oauth2AuthMethodIn
from .oauth_2_grant_type_in import Oauth2GrantTypeIn
from .oauth_jws_signing_algorithm import OauthJwsSigningAlgorithm
from .one_time_token_in import OneTimeTokenIn
from .one_time_token_out import OneTimeTokenOut
from .ordering import Ordering
from .recover_in import RecoverIn
from .recover_out import RecoverOut
from .replay_in import ReplayIn
from .replay_out import ReplayOut
from .retry_schedule_in_out import RetryScheduleInOut
from .rotated_url_out import RotatedUrlOut
from .s3_config import S3Config
from .settings_in import SettingsIn
from .settings_out import SettingsOut
from .sink_http_config import SinkHttpConfig
from .sink_in_type_0 import SinkInType0
from .sink_in_type_0_type import SinkInType0Type
from .sink_in_type_1 import SinkInType1
from .sink_in_type_1_type import SinkInType1Type
from .sink_in_type_2 import SinkInType2
from .sink_in_type_2_type import SinkInType2Type
from .sink_in_type_3 import SinkInType3
from .sink_in_type_3_type import SinkInType3Type
from .sink_in_type_4 import SinkInType4
from .sink_in_type_4_type import SinkInType4Type
from .sink_otel_v1_config import SinkOtelV1Config
from .sink_out_type_0 import SinkOutType0
from .sink_out_type_0_type import SinkOutType0Type
from .sink_out_type_1 import SinkOutType1
from .sink_out_type_1_type import SinkOutType1Type
from .sink_out_type_2 import SinkOutType2
from .sink_out_type_2_type import SinkOutType2Type
from .sink_out_type_3 import SinkOutType3
from .sink_out_type_3_type import SinkOutType3Type
from .sink_out_type_4 import SinkOutType4
from .sink_out_type_4_type import SinkOutType4Type
from .sink_payload_format import SinkPayloadFormat
from .sink_status import SinkStatus
from .sink_status_in import SinkStatusIn
from .sink_transform_in import SinkTransformIn
from .sink_transformation_out import SinkTransformationOut
from .statistics_period import StatisticsPeriod
from .status_code_class import StatusCodeClass
from .stream_in import StreamIn
from .stream_out import StreamOut
from .stream_patch import StreamPatch
from .stream_sink_in_type_0 import StreamSinkInType0
from .stream_sink_in_type_0_type import StreamSinkInType0Type
from .stream_sink_in_type_1 import StreamSinkInType1
from .stream_sink_in_type_1_type import StreamSinkInType1Type
from .stream_sink_in_type_2 import StreamSinkInType2
from .stream_sink_in_type_2_type import StreamSinkInType2Type
from .stream_sink_out_type_0 import StreamSinkOutType0
from .stream_sink_out_type_0_type import StreamSinkOutType0Type
from .stream_sink_out_type_1 import StreamSinkOutType1
from .stream_sink_out_type_1_type import StreamSinkOutType1Type
from .stream_sink_out_type_2 import StreamSinkOutType2
from .stream_sink_out_type_2_type import StreamSinkOutType2Type
from .stream_sink_patch_type_0 import StreamSinkPatchType0
from .stream_sink_patch_type_0_type import StreamSinkPatchType0Type
from .stream_sink_patch_type_1 import StreamSinkPatchType1
from .stream_sink_patch_type_1_type import StreamSinkPatchType1Type
from .stream_sink_patch_type_2 import StreamSinkPatchType2
from .stream_sink_patch_type_2_type import StreamSinkPatchType2Type
from .template_in import TemplateIn
from .template_out import TemplateOut
from .template_patch import TemplatePatch
from .template_update import TemplateUpdate
from .transformation_http_method import TransformationHttpMethod
from .transformation_simulate_in import TransformationSimulateIn
from .transformation_simulate_in_payload import TransformationSimulateInPayload
from .transformation_simulate_out import TransformationSimulateOut
from .transformation_template_kind import TransformationTemplateKind
from .validation_error import ValidationError

__all__ = (
    "AggregateEventTypesOut",
    "ApplicationIn",
    "ApplicationInMetadata",
    "ApplicationOut",
    "ApplicationOutMetadata",
    "ApplicationPatch",
    "ApplicationPatchMetadata",
    "ApplicationStats",
    "ApplicationTokenExpireIn",
    "AppPortalAccessIn",
    "AppPortalAccessOut",
    "AppUsageStatsIn",
    "AppUsageStatsOut",
    "AttemptStatisticsData",
    "AttemptStatisticsResponse",
    "AuthTokenOut",
    "BackgroundTaskData",
    "BackgroundTaskOut",
    "BackgroundTaskStatus",
    "BackgroundTaskType",
    "BorderRadiusConfig",
    "BorderRadiusEnum",
    "ClientSecretJwtParamsIn",
    "CompletionChoice",
    "CompletionMessage",
    "CreateMessageTokenIn",
    "CreateStreamIn",
    "CreateStreamOut",
    "CustomColorPalette",
    "CustomStringsOverride",
    "CustomThemeOverride",
    "DashboardAccessOut",
    "Duration",
    "EndpointCreatedEvent",
    "EndpointCreatedEventData",
    "EndpointCreatedEventType",
    "EndpointDeletedEvent",
    "EndpointDeletedEventData",
    "EndpointDeletedEventType",
    "EndpointDisabledEvent",
    "EndpointDisabledEventData",
    "EndpointDisabledEventType",
    "EndpointHeadersIn",
    "EndpointHeadersInHeaders",
    "EndpointHeadersOut",
    "EndpointHeadersOutHeaders",
    "EndpointHeadersPatchIn",
    "EndpointHeadersPatchInHeaders",
    "EndpointIn",
    "EndpointInMetadata",
    "EndpointMessageOut",
    "EndpointMessageOutPayload",
    "EndpointMtlsConfigIn",
    "EndpointOauthConfigIn",
    "EndpointOauthConfigInExtraParams",
    "EndpointOut",
    "EndpointOutMetadata",
    "EndpointPatch",
    "EndpointPatchMetadata",
    "EndpointSecretOut",
    "EndpointSecretRotateIn",
    "EndpointStats",
    "EndpointTransformationIn",
    "EndpointTransformationOut",
    "EndpointTransformationSimulateIn",
    "EndpointTransformationSimulateInPayload",
    "EndpointTransformationSimulateOut",
    "EndpointUpdate",
    "EndpointUpdatedEvent",
    "EndpointUpdatedEventData",
    "EndpointUpdatedEventType",
    "EndpointUpdateMetadata",
    "EnvironmentIn",
    "EnvironmentOut",
    "EnvironmentSettingsOut",
    "EventExampleIn",
    "EventIn",
    "EventOut",
    "EventStreamOut",
    "EventTypeExampleOut",
    "EventTypeExampleOutExample",
    "EventTypeExampleOutExampleAdditionalProperty",
    "EventTypeFromOpenApi",
    "EventTypeFromOpenApiSchemas",
    "EventTypeFromOpenApiSchemasAdditionalProperty",
    "EventTypeImportOpenApiIn",
    "EventTypeImportOpenApiInSpec",
    "EventTypeImportOpenApiInSpecAdditionalProperty",
    "EventTypeImportOpenApiOut",
    "EventTypeImportOpenApiOutData",
    "EventTypeIn",
    "EventTypeInSchemas",
    "EventTypeInSchemasAdditionalProperty",
    "EventTypeOut",
    "EventTypeOutSchemas",
    "EventTypeOutSchemasAdditionalProperty",
    "EventTypePatch",
    "EventTypePatchSchemas",
    "EventTypePatchSchemasAdditionalProperty",
    "EventTypeSchemaIn",
    "EventTypeSchemaInSchema",
    "EventTypeUpdate",
    "EventTypeUpdateSchemas",
    "EventTypeUpdateSchemasAdditionalProperty",
    "ExportEventTypeOut",
    "FontSizeConfig",
    "GenerateIn",
    "GenerateOut",
    "HttpError",
    "HTTPValidationError",
    "HubspotOauthConfigIn",
    "InboundPathParams",
    "IncomingWebhookPayloadOut",
    "IntegrationIn",
    "IntegrationKeyOut",
    "IntegrationOut",
    "IntegrationUpdate",
    "KafkaSecurityProtocolType",
    "ListResponseApplicationOut",
    "ListResponseApplicationStats",
    "ListResponseBackgroundTaskOut",
    "ListResponseEndpointMessageOut",
    "ListResponseEndpointOut",
    "ListResponseEventTypeOut",
    "ListResponseIntegrationOut",
    "ListResponseMessageAttemptEndpointOut",
    "ListResponseMessageAttemptOut",
    "ListResponseMessageEndpointOut",
    "ListResponseMessageOut",
    "ListResponseSinkOut",
    "ListResponseStreamOut",
    "ListResponseStreamSinkOut",
    "ListResponseTemplateOut",
    "MessageAttemptEndpointOut",
    "MessageAttemptExhaustedEvent",
    "MessageAttemptExhaustedEventData",
    "MessageAttemptExhaustedEventType",
    "MessageAttemptFailedData",
    "MessageAttemptFailingEvent",
    "MessageAttemptFailingEventData",
    "MessageAttemptFailingEventType",
    "MessageAttemptHeadersOut",
    "MessageAttemptHeadersOutSentHeaders",
    "MessageAttemptOut",
    "MessageAttemptRecoveredEvent",
    "MessageAttemptRecoveredEventData",
    "MessageAttemptRecoveredEventType",
    "MessageAttemptTriggerType",
    "MessageBroadcastIn",
    "MessageBroadcastInPayload",
    "MessageBroadcastOut",
    "MessageEndpointOut",
    "MessageEventsOut",
    "MessageIn",
    "MessageInPayload",
    "MessageInTransformationsParams",
    "MessageOut",
    "MessageOutPayload",
    "MessageRawPayloadOut",
    "MessageStatus",
    "MessageSubscriberAuthTokenOut",
    "Oauth2AuthMethodIn",
    "Oauth2GrantTypeIn",
    "OauthJwsSigningAlgorithm",
    "OAuthPayloadIn",
    "OAuthPayloadOut",
    "OneTimeTokenIn",
    "OneTimeTokenOut",
    "Ordering",
    "RecoverIn",
    "RecoverOut",
    "ReplayIn",
    "ReplayOut",
    "RetryScheduleInOut",
    "RotatedUrlOut",
    "S3Config",
    "SettingsIn",
    "SettingsOut",
    "SinkHttpConfig",
    "SinkInType0",
    "SinkInType0Type",
    "SinkInType1",
    "SinkInType1Type",
    "SinkInType2",
    "SinkInType2Type",
    "SinkInType3",
    "SinkInType3Type",
    "SinkInType4",
    "SinkInType4Type",
    "SinkOtelV1Config",
    "SinkOutType0",
    "SinkOutType0Type",
    "SinkOutType1",
    "SinkOutType1Type",
    "SinkOutType2",
    "SinkOutType2Type",
    "SinkOutType3",
    "SinkOutType3Type",
    "SinkOutType4",
    "SinkOutType4Type",
    "SinkPayloadFormat",
    "SinkStatus",
    "SinkStatusIn",
    "SinkTransformationOut",
    "SinkTransformIn",
    "StatisticsPeriod",
    "StatusCodeClass",
    "StreamIn",
    "StreamOut",
    "StreamPatch",
    "StreamSinkInType0",
    "StreamSinkInType0Type",
    "StreamSinkInType1",
    "StreamSinkInType1Type",
    "StreamSinkInType2",
    "StreamSinkInType2Type",
    "StreamSinkOutType0",
    "StreamSinkOutType0Type",
    "StreamSinkOutType1",
    "StreamSinkOutType1Type",
    "StreamSinkOutType2",
    "StreamSinkOutType2Type",
    "StreamSinkPatchType0",
    "StreamSinkPatchType0Type",
    "StreamSinkPatchType1",
    "StreamSinkPatchType1Type",
    "StreamSinkPatchType2",
    "StreamSinkPatchType2Type",
    "TemplateIn",
    "TemplateOut",
    "TemplatePatch",
    "TemplateUpdate",
    "TransformationHttpMethod",
    "TransformationSimulateIn",
    "TransformationSimulateInPayload",
    "TransformationSimulateOut",
    "TransformationTemplateKind",
    "ValidationError",
)



# TODO - remove this special case when we fix the generated code for empty openapi structs
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.application_in import ApplicationIn
    from ..models.message_in_payload import MessageInPayload
    from ..models.message_in_transformations_params import MessageInTransformationsParams


T = TypeVar("T", bound="MessageIn")


@attr.s(auto_attribs=True)
class MessageIn:
    """
    Attributes:
        event_type (str): The event type's name Example: user.signup.
        payload (MessageInPayload): JSON payload to send as the request body of the webhook.

            We also support sending non-JSON payloads. Please contact us for more information. Example: {'email':
            'test@example.com', 'type': 'user.created', 'username': 'test_user'}.
        application (Union[Unset, ApplicationIn]):
        channels (Union[Unset, None, List[str]]): List of free-form identifiers that endpoints can filter by Example:
            ['project_123', 'group_2'].
        event_id (Union[Unset, None, str]): Optional unique identifier for the message Example: unique-msg-identifier.
        payload_retention_hours (Union[Unset, None, int]): Optional number of hours to retain the message payload. Note
            that this is mutually exclusive with `payloadRetentionPeriod`.
        payload_retention_period (Union[Unset, None, int]): Optional number of days to retain the message payload.
            Defaults to 90. Note that this is mutually exclusive with `payloadRetentionHours`. Default: 90. Example: 90.
        tags (Union[Unset, None, List[str]]): List of free-form tags that can be filtered by when listing messages
            Example: ['my_tag', 'other'].
        transformations_params (Union[Unset, None, MessageInTransformationsParams]): Extra parameters to pass to
            Transformations (for future use)
    """

    event_type: str
    payload: "MessageInPayload"
    application: Union[Unset, "ApplicationIn"] = UNSET
    channels: Union[Unset, None, List[str]] = UNSET
    event_id: Union[Unset, None, str] = UNSET
    payload_retention_hours: Union[Unset, None, int] = UNSET
    payload_retention_period: Union[Unset, None, int] = 90
    tags: Union[Unset, None, List[str]] = UNSET
    transformations_params: Union[Unset, None, "MessageInTransformationsParams"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        event_type = self.event_type
        payload = self.payload
        application: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.application, Unset):
            application = self.application.to_dict()

        channels: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.channels, Unset):
            if self.channels is None:
                channels = None
            else:
                channels = self.channels

        event_id = self.event_id
        payload_retention_hours = self.payload_retention_hours
        payload_retention_period = self.payload_retention_period
        tags: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.tags, Unset):
            if self.tags is None:
                tags = None
            else:
                tags = self.tags

        transformations_params = self.transformations_params

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "eventType": event_type,
                "payload": payload,
            }
        )
        if application is not UNSET:
            field_dict["application"] = application
        if channels is not UNSET:
            field_dict["channels"] = channels
        if event_id is not UNSET:
            field_dict["eventId"] = event_id
        if payload_retention_hours is not UNSET:
            field_dict["payloadRetentionHours"] = payload_retention_hours
        if payload_retention_period is not UNSET:
            field_dict["payloadRetentionPeriod"] = payload_retention_period
        if tags is not UNSET:
            field_dict["tags"] = tags
        if transformations_params is not UNSET:
            field_dict["transformationsParams"] = transformations_params

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.application_in import ApplicationIn

        d = src_dict.copy()
        event_type = d.pop("eventType")

        payload = d.pop("payload")

        _application = d.pop("application", UNSET)
        application: Union[Unset, ApplicationIn]
        if isinstance(_application, Unset):
            application = UNSET
        else:
            application = ApplicationIn.from_dict(_application)

        channels = cast(List[str], d.pop("channels", UNSET))

        event_id = d.pop("eventId", UNSET)

        payload_retention_hours = d.pop("payloadRetentionHours", UNSET)

        payload_retention_period = d.pop("payloadRetentionPeriod", UNSET)

        tags = cast(List[str], d.pop("tags", UNSET))

        transformations_params = d.pop("transformationsParams", UNSET)

        message_in = cls(
            event_type=event_type,
            payload=payload,
            application=application,
            channels=channels,
            event_id=event_id,
            payload_retention_hours=payload_retention_hours,
            payload_retention_period=payload_retention_period,
            tags=tags,
            transformations_params=transformations_params,
        )

        message_in.additional_properties = d
        return message_in

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import asyncio
import datetime
import random
from http import HTTPStatus
from time import sleep
from typing import Any, Dict, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.http_error import HttpError
from ...models.http_validation_error import HTTPValidationError
from ...models.list_response_application_stats import ListResponseApplicationStats
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: AuthenticatedClient,
    since: datetime.datetime,
    until: datetime.datetime,
    limit: Union[Unset, None, int] = 50,
    iterator: Union[Unset, None, str] = UNSET,
) -> Dict[str, Any]:
    url = "{}/api/v1/app/stats/usage".format(client.base_url)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    json_since = since.isoformat()

    params["since"] = json_since

    json_until = until.isoformat()

    params["until"] = json_until

    params["limit"] = limit

    params["iterator"] = iterator

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "follow_redirects": client.follow_redirects,
        "params": params,
    }


def _parse_response(*, client: Client, response: httpx.Response) -> ListResponseApplicationStats:
    if response.status_code == HTTPStatus.OK:
        response_200 = ListResponseApplicationStats.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.BAD_REQUEST:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.FORBIDDEN:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.NOT_FOUND:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.CONFLICT:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.UNPROCESSABLE_ENTITY:
        raise HTTPValidationError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.TOO_MANY_REQUESTS:
        raise HttpError.init_exception(response.json(), response.status_code)
    raise errors.UnexpectedStatus(response.status_code, response.content)


def _build_response(*, client: Client, response: httpx.Response) -> Response[ListResponseApplicationStats]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def request_sync_detailed(
    *,
    client: AuthenticatedClient,
    since: datetime.datetime,
    until: datetime.datetime,
    limit: Union[Unset, None, int] = 50,
    iterator: Union[Unset, None, str] = UNSET,
) -> Response[ListResponseApplicationStats]:
    """Get App Usage Stats

     Get basic statistics for all applications.

    Args:
        since (datetime.datetime): Filter the range to data after this date
        until (datetime.datetime): Filter the range to data before this date
        limit (Union[Unset, None, int]): Limit the number of returned items Default: 50.
        iterator (Union[Unset, None, str]): The iterator to use (depends on the chosen ordering)
            Example: app_1srOrx2ZWZBpBUvZwXKQmoEYga2.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ListResponseApplicationStats]
    """

    kwargs = _get_kwargs(
        client=client,
        since=since,
        until=until,
        limit=limit,
        iterator=iterator,
    )

    kwargs["headers"] = {"svix-req-id": f"{random.getrandbits(32)}", **kwargs["headers"]}

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )
    for retry_count, sleep_time in enumerate(client.retry_schedule):
        if response.status_code < 500:
            break

        sleep(sleep_time)
        kwargs["headers"]["svix-retry-count"] = str(retry_count)
        response = httpx.request(
            verify=client.verify_ssl,
            **kwargs,
        )

    return _build_response(client=client, response=response)


def request_sync(
    *,
    client: AuthenticatedClient,
    since: datetime.datetime,
    until: datetime.datetime,
    limit: Union[Unset, None, int] = 50,
    iterator: Union[Unset, None, str] = UNSET,
) -> ListResponseApplicationStats:
    """Get App Usage Stats

     Get basic statistics for all applications.

    Args:
        since (datetime.datetime): Filter the range to data after this date
        until (datetime.datetime): Filter the range to data before this date
        limit (Union[Unset, None, int]): Limit the number of returned items Default: 50.
        iterator (Union[Unset, None, str]): The iterator to use (depends on the chosen ordering)
            Example: app_1srOrx2ZWZBpBUvZwXKQmoEYga2.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ListResponseApplicationStats
    """

    return request_sync_detailed(
        client=client,
        since=since,
        until=until,
        limit=limit,
        iterator=iterator,
    ).parsed


async def request_asyncio_detailed(
    *,
    client: AuthenticatedClient,
    since: datetime.datetime,
    until: datetime.datetime,
    limit: Union[Unset, None, int] = 50,
    iterator: Union[Unset, None, str] = UNSET,
) -> Response[ListResponseApplicationStats]:
    """Get App Usage Stats

     Get basic statistics for all applications.

    Args:
        since (datetime.datetime): Filter the range to data after this date
        until (datetime.datetime): Filter the range to data before this date
        limit (Union[Unset, None, int]): Limit the number of returned items Default: 50.
        iterator (Union[Unset, None, str]): The iterator to use (depends on the chosen ordering)
            Example: app_1srOrx2ZWZBpBUvZwXKQmoEYga2.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ListResponseApplicationStats]
    """

    kwargs = _get_kwargs(
        client=client,
        since=since,
        until=until,
        limit=limit,
        iterator=iterator,
    )

    kwargs["headers"] = {"svix-req-id": f"{random.getrandbits(32)}", **kwargs["headers"]}

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

        for retry_count, sleep_time in enumerate(client.retry_schedule):
            if response.status_code < 500:
                break

            await asyncio.sleep(sleep_time)
            kwargs["headers"]["svix-retry-count"] = str(retry_count)
            response = await _client.request(**kwargs)

    return _build_response(client=client, response=response)


async def request_asyncio(
    *,
    client: AuthenticatedClient,
    since: datetime.datetime,
    until: datetime.datetime,
    limit: Union[Unset, None, int] = 50,
    iterator: Union[Unset, None, str] = UNSET,
) -> ListResponseApplicationStats:
    """Get App Usage Stats

     Get basic statistics for all applications.

    Args:
        since (datetime.datetime): Filter the range to data after this date
        until (datetime.datetime): Filter the range to data before this date
        limit (Union[Unset, None, int]): Limit the number of returned items Default: 50.
        iterator (Union[Unset, None, str]): The iterator to use (depends on the chosen ordering)
            Example: app_1srOrx2ZWZBpBUvZwXKQmoEYga2.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ListResponseApplicationStats
    """

    return (
        await request_asyncio_detailed(
            client=client,
            since=since,
            until=until,
            limit=limit,
            iterator=iterator,
        )
    ).parsed
